#include "define.h"
#include <EEPROMex.h>

//ENCODER
 #include <Encoder.h>
volatile int oldPosition  = 0;
volatile int actualPosition = 0;
volatile int newPosition = 0;
 Encoder myEnc(6, 5);


//LCD / SCK/SCLK  - Pin 12, MOSI/DN   - Pin 11, DC/ D/C   - Pin 10, RST       - Pin 9, CS/SCE    - Pin 8
#include <LCD5110_Graph.h>
LCD5110 myGLCD(12,11,10,9,8);
extern uint8_t TinyFont[];

//RTC // SDA 20 // SCL 21
#include "RTClib.h"
RTC_DS3231 rtc;

//SD / MOSI - pin 51, MISO - pin 50, CLK/SCK - pin 52, CS - pin 42
#include <SPI.h>
#include <SD.h>
File myFile;
char st[]="";

//AnalogMux
#include "analogmuxdemux.h"
AnalogMux amx(M_S0,M_S1,M_S2, S_S0,S_S1,S_S2, READPIN);
unsigned long timerAnalogRead = 0;
int anRead[64];

//chceckButton() globals
volatile unsigned long lastDebounceTime = 0; 
volatile int debounceDelay = 50;
volatile bool button_state = 0;
volatile bool last_button_state = 0;
volatile bool b_pressed = 0;
volatile bool b_l_pressed = 0;
volatile bool b_clicked = 0;
volatile bool reading = 0;

//ARRAYS
const int VINPIN[] = {0,VIN1,VIN2,VIN3,VIN4,VIN5,VIN6,VIN7,VIN8,VIN9,VIN10,VIN11,VIN12,VIN13,VIN14,VIN15,VIN16,VIN17,VIN18,VIN19,VIN20};
const int VOUTPIN[] = {0,VOUT1,VOUT2,VOUT3,VOUT4,VOUT5,VOUT6,VOUT7,VOUT8,VOUT9,VOUT10,VOUT11,VOUT12,VOUT13,VOUT14,VOUT15,VOUT16,VOUT17,VOUT18,VOUT19,VOUT20};
const int TEMPPIN[] = {0,TEMP1,TEMP2,TEMP3,TEMP4,TEMP5,TEMP6,TEMP7,TEMP8,TEMP9,TEMP10,TEMP11,TEMP12,TEMP13,TEMP14,TEMP15,TEMP16,TEMP17,TEMP18,TEMP19,TEMP20};
const int BATPIN[] = {0,BAT1,BAT2,BAT3,BAT4,BAT5,BAT6,BAT7,BAT8,BAT9,BAT10,BAT11,BAT12,BAT13,BAT14,BAT15,BAT16,BAT17,BAT18,BAT19,BAT20};
const float RESVAL[] = {0,RES1,RES2,RES3,RES4,RES5,RES6,RES7,RES8,RES9,RES10,RES11,RES12,RES13,RES14,RES15,RES16,RES17,RES18,RES19,RES20};
int BATSTAT[] = {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
const char BATSTATSTR[4][3] = {"--","OK","LV","HV"};
bool actionDone[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
float mah[] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
float intRes[] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
unsigned long timerDisStat[] = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};


//REST GLOBALS

//ACTION SELECT
bool menuActive = false;
int action = 0;
bool actionActive = false;
bool askCont = false;
unsigned long timerAskCont = 0;

//DO ACTION
bool charging = false;
bool discharging = false;
bool storageCharging = false;
bool chargingComplete = false;
bool dischargingComplete = false;
unsigned long timerAction = 0;
unsigned long timerCharging = 0;
unsigned long timerDischarging = 0;
int intResCount = 0;
bool firstDischEnter = true;

float incomingByte = 0;

//FUNCTIONS
void screenRefresh();
void setStatus();
void readButton();
void readSensors();
void readEncoder();
void displayTEMP();
void displayVIN();
void displayVOUT();
void displayVRES();
void displayBATSTAT();
void displayMENU();
void displayMAH();
double Thermistor();
void buttonAction();
void askContinue();
void doAction();
void charge();
void chargerON();
void chargerOFF();
int countBatStatOK();
void discharge();
void continueDischarge();
void stopDischarge();
void intResTest();
void checkTemp();



void setup() 
 {
    Serial.begin(9600); // Open serial communications and wait for port to open:
    while (!Serial) 
      {
       ; // wait for serial port to connect. Needed for native USB port only
      }

    //LCD Init
    myGLCD.InitLCD();
    myGLCD.setFont(TinyFont);
    pinMode(13,OUTPUT); //LED
    analogWrite(13,255); // LED ON
    myGLCD.clrScr();
    Serial.println("LCD OK");
    myGLCD.print("LCD OK", CENTER,0);
    myGLCD.update();
  
    //RTC INIT
    if (! rtc.begin()) 
    {
     Serial.println("Couldn't find RTC");
     myGLCD.print("Couldn't find RTC", CENTER,6);
     myGLCD.update();
     while (1);
    }
     Serial.println("RTC OK");
     myGLCD.print("RTC OK", CENTER,6);
     myGLCD.update();
    
    if (rtc.lostPower()) 
    {
     Serial.println("RTC lost power, lets set the time!");
     myGLCD.print("RTC lost power", CENTER,6);
     myGLCD.update();
     rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));
    }
  
    //SD INIT
    if (!SD.begin(42)) 
    {
      Serial.println("SD FAIL!");
      myGLCD.print("SD FAIL", CENTER,12);
      myGLCD.update();
      return;
    } else 
       {
       Serial.println("SD OK");
        myGLCD.print("SD OK", CENTER,12);
        myGLCD.update();
       }
  
      pinMode(BUTTON, INPUT);
      pinMode(BUZZER, OUTPUT);
      pinMode(RELAY, OUTPUT);
      digitalWrite(RELAY,HIGH);

      for(int i=1;i<=20;i++)
      pinMode(BATPIN[i], OUTPUT);

    
      //BUZZER TEST
      digitalWrite(BUZZER,HIGH);
      delay(250);
      digitalWrite(BUZZER, LOW);

       myGLCD.print(".:STACJA LI-ION:.", CENTER,36);
       myGLCD.print(__DATE__, CENTER,42);
       myGLCD.update();
  
}

void loop() {

  

if (Serial.available() > 0) {
                // read the incoming byte:
                incomingByte = Serial.parseInt();

                // say what you got:
                Serial.print("I received: ");
                Serial.println(incomingByte, DEC);

       
}


 readEncoder();
 readButton();
 buttonAction();



 if(millis()-timerAnalogRead > 500){
 readSensors();
 checkTemp();
 if(!actionActive)
 setStatus();
 screenRefresh(); }

  if((millis()-timerAction) > ACTION_INTERVAL){
  doAction();
  timerAction = millis();}

}

void doAction(){

  //CHARGE + CAPACITY TEST + STORAGE CHARGE
  if (actionActive && action == MENU_CHRGQTSTSTRG)
  {
    //Serial.println("No to lecimy z programem 1");
   if(!charging && !discharging && !chargingComplete && !dischargingComplete)
   {
    for (int i=1;i<=20;i++){
    mah[i]=0.0;
    if(BATSTAT[i] == 1)
    actionDone[i]=0;
    else
    actionDone[i]=1;}
    intResCount=0;
    
     //JEZELI SA DOBRE OGNIWA ROZPOCZNIJ LADOWANIE
      if(countBatStatOK() > 0)
      {
       charging = true;
       timerCharging = millis();
       discharging = false;
      }
    } 

       if(!charging && !discharging && !chargingComplete && dischargingComplete)
   {
     //JEZELI SA DOBRE OGNIWA ROZPOCZNIJ LADOWANIE
      if(countBatStatOK() > 0)
      {
       for (int i=1;i<=20;i++){
       if(BATSTAT[i] == 1)
        actionDone[i]=0;
        else
        actionDone[i]=1;}

      digitalWrite(BUZZER,HIGH);
      delay(1000);
      digitalWrite(BUZZER, LOW);
       
       charging = true;
       timerCharging = millis();
       discharging = false;
       storageCharging = true;
       
      }
    } 



    if(!charging && !discharging && chargingComplete && !storageCharging)
    {
    for (int i=1;i<=20;i++){
    if(BATSTAT[i] == 1)
    actionDone[i]=0;
    else
    actionDone[i]=1;}

      digitalWrite(BUZZER,HIGH);
      delay(1000);
      digitalWrite(BUZZER, LOW);
    discharging = true;
    timerDischarging = millis();
    chargingComplete = false;
    intResTest();
    timerDischarging = millis();
    } else if(!charging && !discharging && chargingComplete && storageCharging)
    {
      chargingComplete = false;
      dischargingComplete = false;
      storageCharging = false;
      actionActive = false;
      charging = false;
      discharging = false;
      action = 0;


    }
 





    if(charging && !discharging && !storageCharging)
    charge(FULLY_CHARGED);
    

    if(charging && !discharging && storageCharging)
    charge(STORAGE);
    

    if(discharging && !charging)
    discharge();
    
}
  //BRAK AKCJI, AKCJA ANULOWANA
  if(!actionActive)
  {
    //action = 0;
    chargerOFF();
    charging = false;
    discharging = false;
    firstDischEnter = true;
    chargingComplete = false;
    dischargingComplete = false;
    storageCharging = false;
    for(int i=1;i<=20;i++)
    digitalWrite(BATPIN[i], LOW);
    timerAction = millis();
  }
}

void intResTest(){
  intResCount++;
  float vin[21];
  float vin_load[21];
  Serial.println("Internal Resistance");
  for(int i=1;i<=20;i++)
  {
   if(BATSTAT[i]==1)
   { 
     readSensors();
     vin[i] = VREF/1023*anRead[VINPIN[i]];
     digitalWrite(BATPIN[i],HIGH);
   
     delay(5000);
     readSensors(); 
     
     vin_load[i] = VREF/1023*anRead[VINPIN[i]];
     digitalWrite(BATPIN[i],LOW);
     intRes[i] = (intRes[i] + (vin[i]-vin_load[i])/RESVAL[i]);
     Serial.print(i);
     Serial.print("\t");
     Serial.print(vin[i]);
     Serial.print("\t");
     Serial.print(vin_load[i]);
     Serial.print("\t");
     Serial.println(intRes[i]/intResCount);
   }
  }
}

void discharge(){
  int countNotDischarged = 0;
  float VIN = 0;
  float VOUT = 0;
  for(int i=1;i<=20;i++)
  {
   if(BATSTAT[i]==1)
   { 
    VIN = VREF/1023*anRead[VINPIN[i]];
    VOUT = (VREF/1023*anRead[VOUTPIN[i]]);
    //VOUT = 0.2;
    if(((VIN-VOUT) < DISCHARGED) && ((VIN-VOUT) > 1.0))
    actionDone[i] = 1;
   }
  }

  for(int i=1;i<=20;i++)
  {
    if(actionDone[i] == 0)  
    {
    countNotDischarged++;
    continueDischarge(i);
    } else
    stopDischarge(i);
   }
  

    if (countNotDischarged == 0)
  {
    discharging = false;
    dischargingComplete = true;
    firstDischEnter = true;
    intResTest();
    /*DateTime now = rtc.now();
    Serial.println("Podsumowanie rozladowania:");
    myFile = SD.open("test.txt", FILE_WRITE);
    if (myFile) {
      myFile.println("Podsumowanie rozladowania:");
        } else {
    Serial.println("error opening test.txt");
  }
     for(int i=1;i<=20;i++)
  {
   if(BATSTAT[i]==1)
   { 
      
      Serial.print(i);
      Serial.print("\t");
      Serial.print(String(now.day()) + "-" + String(now.month()) + "-" + String(now.year()) + "_"  + String(now.hour()) + ":" + String(now.minute()));
      Serial.print("\t");
      Serial.println(mah[i]);



    if (myFile) {
    myFile.print(i);
    myFile.print("\t");
    myFile.print(String(now.day()) + "-" + String(now.month()) + "-" + String(now.year()) + "_"  + String(now.hour()) + ":" + String(now.minute()));
    myFile.print("\t");
    myFile.println(mah[i]);
    
    } else {
    Serial.println("error opening test.txt");
  }
  }
  }
  myFile.close();*/
  }
  
}

void charge(float voltage){
  long timer = 10000;
  int countNotCharged = 0;
  float VIN = 0.0;
  for(int i=1;i<=20;i++)
  {
   if(BATSTAT[i]==1)
   { 
    VIN = VREF/1023*anRead[VINPIN[i]];
    if(VIN > voltage)
    actionDone[i] = 1;        
   }
  }

  for (int i=1;i<=20;i++){
  if(actionDone[i] == 0)
  countNotCharged++;}
  
  if (countNotCharged > 0 && !chargingComplete)
  {
    chargerON();
    timerCharging = millis();
  } else 
  {
    chargingComplete = true;
    timer = ((TIME_FULL_CHARGING - ((long)millis() - (long)timerCharging))/1000L);
    Serial.print("[s] do wylaczenia ladowarki: ");
    Serial.println(timer);

    if((((millis() - timerCharging) > TIME_FULL_CHARGING) && !storageCharging) || incomingByte == 111) 
    {
    chargerOFF();
    charging = false;
    }

    if(storageCharging) 
    {
    chargerOFF();
    chargingComplete = true;
    charging = false;
    intResTest();

    DateTime now = rtc.now();
    Serial.println("Podsumowanie testu:");
    myFile = SD.open("test.txt", FILE_WRITE);
    if (myFile) {
      myFile.println("Podsumowanie testu:");
        } else {
    Serial.println("error opening test.txt");
  }
     int act_serial_number = 1;
     for(int i=1;i<=20;i++)
  {
   if(BATSTAT[i]==1)
   { 
      
      Serial.print(i);
      Serial.print("\t");
      Serial.print(EEPROM.readInt(0)+act_serial_number);
      Serial.print("\t");
      Serial.print(String(now.day()) + "-" + String(now.month()) + "-" + String(now.year()) + "_"  + String(now.hour()) + ":" + String(now.minute()));
      Serial.print("\t");
      Serial.print(mah[i]);
      Serial.print("\t");
      Serial.println(intRes[i]/intResCount);



    if (myFile) {
    myFile.print(i);
    myFile.print("\t");
    myFile.print(EEPROM.readInt(0)+act_serial_number);
    myFile.print("\t");
    myFile.print(String(now.day()) + "-" + String(now.month()) + "-" + String(now.year()) + "_"  + String(now.hour()) + ":" + String(now.minute()));
    myFile.print("\t");
    myFile.print(mah[i]);
    myFile.print("\t");
    myFile.println(intRes[i]/intResCount);

    act_serial_number++;
    
    } else {
    Serial.println("error opening test.txt");
  }
  }
  }
  myFile.close();
  act_serial_number = EEPROM.readInt(0)+act_serial_number-1;
  EEPROM.writeInt(0,act_serial_number);

      digitalWrite(BUZZER,HIGH);
      delay(1000);
      digitalWrite(BUZZER, LOW);
      delay(1000);
      digitalWrite(BUZZER,HIGH);
      delay(1000);
      digitalWrite(BUZZER, LOW);
      delay(1000);
      digitalWrite(BUZZER,HIGH);
      delay(1000);
      digitalWrite(BUZZER, LOW);
    }
  }
    
    Serial.print("Nienaladowanych ogniw: ");
    Serial.println(countNotCharged);
}

void screenRefresh()
{
 myGLCD.clrScr();

 if(menuActive == false && !askCont)
 {
  switch (actualPosition)
  {
   case 1:
   displayBATSTAT();
   break;
   case 2:
   displayVIN();
   break;
   case 3:
   displayMAH();
   break;
   case 4:
   displayVOUT();
   break;
   case 5:
   displayVRES();
   break;
   case 6:
   displayTEMP();
   break;
   case 7:
   DateTime now = rtc.now();
   myGLCD.print(String(actualPosition),CENTER, 24);
   myGLCD.print(String(now.hour()) + ":" + String(now.minute()) + ":" + String(now.second()), CENTER,30);
   myGLCD.printNumF(EEPROM.readInt(0),0,RIGHT,42);
   myGLCD.update();
   break;
  }
  if(actionActive)
  {
   myGLCD.print("A"+String(action), 0, 42);
   myGLCD.update();
  }
 } 
 
   if (action == 0 && menuActive)
   {
    displayMENU();
   }

   
}

void buttonAction(){

  //NIC SIE NIE DZIEJE , A WYSWIETLANE INFO
   if(!menuActive && b_clicked && !askCont && action == 0 && actualPosition != 7)
   {
    menuActive = true;
    actualPosition = 8;
    b_clicked=false;
   }

     //NIC SIE NIE DZIEJE , A WYSWIETLANE INFO
   if(!menuActive && b_clicked && !askCont && action == 0 && actualPosition == 7)
   {
    int serial = 0;
    myFile = SD.open("test.txt");
    if (myFile) 
    {
     serial = (myFile.readStringUntil('\r')).toInt();
     EEPROM.writeInt(0,serial);
    }
    myFile.close();
    b_clicked=false;
   }

   //TRWA AKCJA, WYSWIETLANE INFO
   if(!menuActive && b_clicked && !askCont && action != 0)
   {
    askContinue();
    b_clicked=false;
   }

   //TRWA AKCJA I ANULOWANO PYTANIE O ODWOLANIE AKCJI
   if((!menuActive && b_clicked && askCont && action != 0) || !menuActive && askCont &&((millis() - timerAskCont) > 5000))
   {
    askCont = false;
    b_clicked=false;
   }

   //TRWA AKCJA I POTWIERDZONO PYTANIE O ODWOLANIE AKCJI
   if(!menuActive && !b_clicked && b_l_pressed && askCont && action != 0)
   {
          //BUZZER TEST
      digitalWrite(BUZZER,HIGH);
      delay(250);
      digitalWrite(BUZZER, LOW);
      askCont = false;
      actionActive = false;
      action = 0;
      menuActive = false;
      b_clicked = false;
      chargerOFF();
     charging = false;
     chargingComplete = false;
     discharging = false;
     dischargingComplete = false;
      //b_l_pressed = false;
   }
   

   //WYSWIETLANE MENU 
   if(menuActive && b_clicked && !askCont)
  {
  
   switch (actualPosition)
   {
    case MENU_CHRGQTSTSTRG:
    action = MENU_CHRGQTSTSTRG;
    askContinue();
    b_clicked = false;
    break;

    case MENU_QUICKINTRES:
    intResTest();
    b_clicked = false;
    break;
    
    case 8: //EXIT MENU
    action=0;
    menuActive = false;
    actualPosition = 1;
    b_clicked = false;
    break;
   }
 }

 //ANULOWANIE WYBORU POZYCJI MENU
 if((menuActive && askCont && b_clicked && !b_l_pressed) || (menuActive && askCont && (millis() - timerAskCont) > 5000))
 {
  askCont = false;
  displayMENU();
  b_clicked = false;
  action = 0;
 }

  //POTWIERDZENIE WYBORU POZYCJI MENU
  if(askCont && !b_clicked && b_l_pressed)
  {
      //BUZZER TEST
      digitalWrite(BUZZER,HIGH);
      delay(250);
      digitalWrite(BUZZER, LOW);
      askCont = false;
      actionActive = true;
      menuActive = false;
      actualPosition = 1;
      b_clicked = false;
      //b_l_pressed = false;
  }
 
}

void setStatus(){
  float vin;
  
  for(int i=1;i<21;i++)
  {

    vin = VREF/1023*anRead[VINPIN[i]];
  
    //NO BATTERY
    if(vin<0.5)
    BATSTAT[i] = 0;

    //OK
    if(vin>=2.5 && vin<4.3)
    BATSTAT[i]= 1;

    //LV
    if(vin>0.5 && vin<2.5)
    BATSTAT[i]= 2;
      
    //HV
    if(vin>4.3)
    BATSTAT[i]= 3;
    
    
  }
}


void readSensors()
{
      for (int pinno=0; pinno< NO_PINS; pinno++) 
      {
        amx.SelectPin(pinno);
        anRead[pinno] = amx.AnalogRead(); 
        //anRead[pinno] = incomingByte;
      }
    
      timerAnalogRead = millis();
}

void readEncoder()
{
     newPosition = myEnc.read();
  if ((newPosition != oldPosition) && ((abs(newPosition - oldPosition) > 3))) 
  {

    if(newPosition > oldPosition)
      actualPosition++;
    else
     actualPosition--;
       
     oldPosition = newPosition;
     
    if(menuActive == false)
    {
     if(actualPosition > 7)
      actualPosition = 1;
     if(actualPosition < 1)
      actualPosition = 7;
    } else
      {
       if(actualPosition > 8)
       actualPosition = 1;
       if(actualPosition < 1)
       actualPosition = 8;
      }
  }
}

void readButton()
{
    reading = digitalRead(BUTTON);
    if (reading != last_button_state) 
     {
      // reset the debouncing timer
      lastDebounceTime = millis();

     }

    if ((millis() - lastDebounceTime) > debounceDelay) 
     {
       
       if (reading != button_state) 
        {
         button_state = reading;
         if (button_state == HIGH) 
          b_pressed = true;

         if (button_state == LOW) 
          {
           if (!b_l_pressed)
           b_clicked = true;
           
           b_pressed = false;
           b_l_pressed = false;
           
          }
        }

       if (reading == button_state && button_state == HIGH)
       {
        if ((millis() - lastDebounceTime) > BUTTON_LONG_TIME && b_l_pressed == 0)
        b_l_pressed = true;
       }
       
     }
  last_button_state = reading;
}

double Thermistor(int RawADC) {
 double Temp;
 Temp = log(10000.0*((1024.0/RawADC-1))); 
 Temp = 1 / (0.001129148 + (0.000234125 + (0.0000000876741 * Temp * Temp ))* Temp );
 Temp = Temp - 273.15;            // Convert Kelvin to Celcius
 return Temp;
}

void displayTEMP()
{
   //myGLCD.drawLine(26,0,26,41);
  //myGLCD.drawLine(27,0,27,41);
  myGLCD.print(":", 7, 0);
  myGLCD.print("01", 0, 0);
  //myGLCD.print("2000", 10, 0);
  myGLCD.printNumF(Thermistor(anRead[TEMP1]),1,10,0);
  
  myGLCD.print(":", 36, 0);
  myGLCD.print("02", 29, 0);
  //myGLCD.print("2000", 39, 0);
  myGLCD.printNumF(Thermistor(anRead[TEMP2]),1,39,0);
  
  myGLCD.print(":", 65, 0);
  myGLCD.print("03", 58, 0);
  //myGLCD.print("2000", 68, 0);
  myGLCD.printNumF(Thermistor(anRead[TEMP3]),1,68,0);
  
  myGLCD.print(":", 7, 6);
  myGLCD.print("04", 0, 6);
  //myGLCD.print("2000", 10, 6);
  myGLCD.printNumF(Thermistor(anRead[TEMP4]),1,10,6);
  
  myGLCD.print(":", 36, 6);
  myGLCD.print("05", 29, 6);
  //myGLCD.print("2000", 39, 6);
  myGLCD.printNumF(Thermistor(anRead[TEMP5]),1,39,6);
  
  myGLCD.print(":", 65, 6);
  myGLCD.print("06", 58, 6);
  //myGLCD.print("2000", 68, 6);
  myGLCD.printNumF(Thermistor(anRead[TEMP6]),1,68,6);
    
  myGLCD.print(":", 7, 12);
  myGLCD.print("07", 0, 12);
  //myGLCD.print("2000", 10, 12);
  myGLCD.printNumF(Thermistor(anRead[TEMP7]),1,10,12);
  
  myGLCD.print(":", 36, 12);
  myGLCD.print("08", 29, 12);
  //myGLCD.print("2000", 39, 12);
  myGLCD.printNumF(Thermistor(anRead[TEMP8]),1,39,12);
  
  myGLCD.print(":", 65, 12);
  myGLCD.print("09", 58, 12);
  //myGLCD.print("2000", 68, 12);
  myGLCD.printNumF(Thermistor(anRead[TEMP9]),1,68,12);

  myGLCD.print(":", 7, 18);
  myGLCD.print("10", 0, 18);
  //myGLCD.print("2000", 10, 18);
  myGLCD.printNumF(Thermistor(anRead[TEMP10]),1,10,18);
  
  myGLCD.print(":", 36, 18);
  myGLCD.print("11", 29, 18);
  //myGLCD.print("2000", 39, 18);
  myGLCD.printNumF(Thermistor(anRead[TEMP11]),1,39,18);
  
  myGLCD.print(":", 65, 18);
  myGLCD.print("12", 58, 18);
  //myGLCD.print("2000", 68, 18);
  myGLCD.printNumF(Thermistor(anRead[TEMP12]),1,68,18);

  myGLCD.print(":", 7, 24);
  myGLCD.print("13", 0, 24);
  //myGLCD.print("2000", 10, 24);
  myGLCD.printNumF(Thermistor(anRead[TEMP13]),1,10,24);
  
  myGLCD.print(":", 36, 24);
  myGLCD.print("14", 29, 24);
  //myGLCD.print("2000", 39, 24);
  myGLCD.printNumF(Thermistor(anRead[TEMP14]),1,39,24);
  
  myGLCD.print(":", 65, 24);
  myGLCD.print("15", 58, 24);
  //myGLCD.print("2000", 68, 24);
  myGLCD.printNumF(Thermistor(anRead[TEMP15]),1,68,24);
  
  myGLCD.print(":", 7, 30);
  myGLCD.print("16", 0, 30);
  //myGLCD.print("2000", 10, 30);
  myGLCD.printNumF(Thermistor(anRead[TEMP16]),1,10,30);
  
  myGLCD.print(":", 36, 30);
  myGLCD.print("17", 29, 30);
  //myGLCD.print("2000", 39, 30);
  myGLCD.printNumF(Thermistor(anRead[TEMP17]),1,39,30);
  
  myGLCD.print(":", 65, 30);
  myGLCD.print("18", 58, 30);
  //myGLCD.print("2000", 68, 30);
  myGLCD.printNumF(Thermistor(anRead[TEMP18]),1,68,30);

  myGLCD.print(":", 7, 36);
  myGLCD.print("19", 0, 36);
  //myGLCD.print("2000", 10, 36);
  myGLCD.printNumF(Thermistor(anRead[TEMP19]),1,10,36);

  myGLCD.print(":", 65, 36);
  myGLCD.print("20", 58, 36);
  //myGLCD.print("2000", 68, 36);
  myGLCD.printNumF(Thermistor(anRead[TEMP20]),1,68,36);
  myGLCD.print("TEMP [deg C]", CENTER,42);
  myGLCD.update();
}

void displayVIN()
{
   //myGLCD.drawLine(26,0,26,41);
  //myGLCD.drawLine(27,0,27,41);
  myGLCD.print(":", 7, 0);
  myGLCD.print("01", 0, 0);
  //myGLCD.print("2000", 10, 0);
  myGLCD.printNumF(VREF/1023*anRead[VIN1],2,10,0);
  
  myGLCD.print(":", 36, 0);
  myGLCD.print("02", 29, 0);
  //myGLCD.print("2000", 39, 0);
  myGLCD.printNumF(VREF/1023*anRead[VIN2],2,39,0);
  
  myGLCD.print(":", 65, 0);
  myGLCD.print("03", 58, 0);
  //myGLCD.print("2000", 68, 0);
  myGLCD.printNumF(VREF/1023*anRead[VIN3],2,68,0);
  
  myGLCD.print(":", 7, 6);
  myGLCD.print("04", 0, 6);
  //myGLCD.print("2000", 10, 6);
  myGLCD.printNumF(VREF/1023*anRead[VIN4],2,10,6);
  
  myGLCD.print(":", 36, 6);
  myGLCD.print("05", 29, 6);
  //myGLCD.print("2000", 39, 6);
  myGLCD.printNumF(VREF/1023*anRead[VIN5],2,39,6);
  
  myGLCD.print(":", 65, 6);
  myGLCD.print("06", 58, 6);
  //myGLCD.print("2000", 68, 6);
  myGLCD.printNumF(VREF/1023*anRead[VIN6],2,68,6);
    
  myGLCD.print(":", 7, 12);
  myGLCD.print("07", 0, 12);
  //myGLCD.print("2000", 10, 12);
  myGLCD.printNumF(VREF/1023*anRead[VIN7],2,10,12);
  
  myGLCD.print(":", 36, 12);
  myGLCD.print("08", 29, 12);
  //myGLCD.print("2000", 39, 12);
  myGLCD.printNumF(VREF/1023*anRead[VIN8],2,39,12);
  
  myGLCD.print(":", 65, 12);
  myGLCD.print("09", 58, 12);
  //myGLCD.print("2000", 68, 12);
  myGLCD.printNumF(VREF/1023*anRead[VIN9],2,68,12);

  myGLCD.print(":", 7, 18);
  myGLCD.print("10", 0, 18);
  //myGLCD.print("2000", 10, 18);
  myGLCD.printNumF(VREF/1023*anRead[VIN10],2,10,18);
  
  myGLCD.print(":", 36, 18);
  myGLCD.print("11", 29, 18);
  //myGLCD.print("2000", 39, 18);
  myGLCD.printNumF(VREF/1023*anRead[VIN11],2,39,18);
  
  myGLCD.print(":", 65, 18);
  myGLCD.print("12", 58, 18);
  //myGLCD.print("2000", 68, 18);
  myGLCD.printNumF(VREF/1023*anRead[VIN12],2,68,18);

  myGLCD.print(":", 7, 24);
  myGLCD.print("13", 0, 24);
  //myGLCD.print("2000", 10, 24);
  myGLCD.printNumF(VREF/1023*anRead[VIN13],2,10,24);
  
  myGLCD.print(":", 36, 24);
  myGLCD.print("14", 29, 24);
  //myGLCD.print("2000", 39, 24);
  myGLCD.printNumF(VREF/1023*anRead[VIN14],2,39,24);
  
  myGLCD.print(":", 65, 24);
  myGLCD.print("15", 58, 24);
  //myGLCD.print("2000", 68, 24);
  myGLCD.printNumF(VREF/1023*anRead[VIN15],2,68,24);
  
  myGLCD.print(":", 7, 30);
  myGLCD.print("16", 0, 30);
  //myGLCD.print("2000", 10, 30);
  myGLCD.printNumF(VREF/1023*anRead[VIN16],2,10,30);
  
  myGLCD.print(":", 36, 30);
  myGLCD.print("17", 29, 30);
  //myGLCD.print("2000", 39, 30);
  myGLCD.printNumF(VREF/1023*anRead[VIN17],2,39,30);
  
  myGLCD.print(":", 65, 30);
  myGLCD.print("18", 58, 30);
  //myGLCD.print("2000", 68, 30);
  myGLCD.printNumF(VREF/1023*anRead[VIN18],2,68,30);

  myGLCD.print(":", 7, 36);
  myGLCD.print("19", 0, 36);
  //myGLCD.print("2000", 10, 36);
  myGLCD.printNumF(VREF/1023*anRead[VIN19],2,10,36);

  myGLCD.print(":", 65, 36);
  myGLCD.print("20", 58, 36);
  //myGLCD.print("2000", 68, 36);
  myGLCD.printNumF(VREF/1023*anRead[VIN20],2,68,36);
  myGLCD.print("VIN [V]", CENTER,42);
  myGLCD.update();
}

void displayVOUT()
{
   //myGLCD.drawLine(26,0,26,41);
  //myGLCD.drawLine(27,0,27,41);
  myGLCD.print(":", 7, 0);
  myGLCD.print("01", 0, 0);
  //myGLCD.print("2000", 10, 0);
  myGLCD.printNumF(VREF/1023*anRead[VOUT1],2,10,0);
  
  myGLCD.print(":", 36, 0);
  myGLCD.print("02", 29, 0);
  //myGLCD.print("2000", 39, 0);
  myGLCD.printNumF(VREF/1023*anRead[VOUT2],2,39,0);
  
  myGLCD.print(":", 65, 0);
  myGLCD.print("03", 58, 0);
  //myGLCD.print("2000", 68, 0);
  myGLCD.printNumF(VREF/1023*anRead[VOUT3],2,68,0);
  
  myGLCD.print(":", 7, 6);
  myGLCD.print("04", 0, 6);
  //myGLCD.print("2000", 10, 6);
  myGLCD.printNumF(VREF/1023*anRead[VOUT4],2,10,6);
  
  myGLCD.print(":", 36, 6);
  myGLCD.print("05", 29, 6);
  //myGLCD.print("2000", 39, 6);
  myGLCD.printNumF(VREF/1023*anRead[VOUT5],2,39,6);
  
  myGLCD.print(":", 65, 6);
  myGLCD.print("06", 58, 6);
  //myGLCD.print("2000", 68, 6);
  myGLCD.printNumF(VREF/1023*anRead[VOUT6],2,68,6);
    
  myGLCD.print(":", 7, 12);
  myGLCD.print("07", 0, 12);
  //myGLCD.print("2000", 10, 12);
  myGLCD.printNumF(VREF/1023*anRead[VOUT7],2,10,12);
  
  myGLCD.print(":", 36, 12);
  myGLCD.print("08", 29, 12);
  //myGLCD.print("2000", 39, 12);
  myGLCD.printNumF(VREF/1023*anRead[VOUT8],2,39,12);
  
  myGLCD.print(":", 65, 12);
  myGLCD.print("09", 58, 12);
  //myGLCD.print("2000", 68, 12);
  myGLCD.printNumF(VREF/1023*anRead[VOUT9],2,68,12);

  myGLCD.print(":", 7, 18);
  myGLCD.print("10", 0, 18);
  //myGLCD.print("2000", 10, 18);
  myGLCD.printNumF(VREF/1023*anRead[VOUT10],2,10,18);
  
  myGLCD.print(":", 36, 18);
  myGLCD.print("11", 29, 18);
  //myGLCD.print("2000", 39, 18);
  myGLCD.printNumF(VREF/1023*anRead[VOUT11],2,39,18);
  
  myGLCD.print(":", 65, 18);
  myGLCD.print("12", 58, 18);
  //myGLCD.print("2000", 68, 18);
  myGLCD.printNumF(VREF/1023*anRead[VOUT12],2,68,18);

  myGLCD.print(":", 7, 24);
  myGLCD.print("13", 0, 24);
  //myGLCD.print("2000", 10, 24);
  myGLCD.printNumF(VREF/1023*anRead[VOUT13],2,10,24);
  
  myGLCD.print(":", 36, 24);
  myGLCD.print("14", 29, 24);
  //myGLCD.print("2000", 39, 24);
  myGLCD.printNumF(VREF/1023*anRead[VOUT14],2,39,24);
  
  myGLCD.print(":", 65, 24);
  myGLCD.print("15", 58, 24);
  //myGLCD.print("2000", 68, 24);
  myGLCD.printNumF(VREF/1023*anRead[VOUT15],2,68,24);
  
  myGLCD.print(":", 7, 30);
  myGLCD.print("16", 0, 30);
  //myGLCD.print("2000", 10, 30);
  myGLCD.printNumF(VREF/1023*anRead[VOUT16],2,10,30);
  
  myGLCD.print(":", 36, 30);
  myGLCD.print("17", 29, 30);
  //myGLCD.print("2000", 39, 30);
  myGLCD.printNumF(VREF/1023*anRead[VOUT17],2,39,30);
  
  myGLCD.print(":", 65, 30);
  myGLCD.print("18", 58, 30);
  //myGLCD.print("2000", 68, 30);
  myGLCD.printNumF(VREF/1023*anRead[VOUT18],2,68,30);

  myGLCD.print(":", 7, 36);
  myGLCD.print("19", 0, 36);
  //myGLCD.print("2000", 10, 36);
  myGLCD.printNumF(VREF/1023*anRead[VOUT19],2,10,36);

  myGLCD.print(":", 65, 36);
  myGLCD.print("20", 58, 36);
  //myGLCD.print("2000", 68, 36);
  myGLCD.printNumF(VREF/1023*anRead[VOUT20],2,68,36);
  myGLCD.print("VOUT [V]", CENTER,42);
  myGLCD.update();
}

void displayVRES()
{
   //myGLCD.drawLine(26,0,26,41);
  //myGLCD.drawLine(27,0,27,41);
  myGLCD.print(":", 7, 0);
  myGLCD.print("01", 0, 0);
  //myGLCD.print("2000", 10, 0);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN1])-(VREF/1023*anRead[VOUT1])),2,10,0);
  
  myGLCD.print(":", 36, 0);
  myGLCD.print("02", 29, 0);
  //myGLCD.print("2000", 39, 0);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN2])-(VREF/1023*anRead[VOUT2])),2,39,0);
  
  myGLCD.print(":", 65, 0);
  myGLCD.print("03", 58, 0);
  //myGLCD.print("2000", 68, 0);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN3])-(VREF/1023*anRead[VOUT3])),2,68,0);
  
  myGLCD.print(":", 7, 6);
  myGLCD.print("04", 0, 6);
  //myGLCD.print("2000", 10, 6);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN4])-(VREF/1023*anRead[VOUT4])),2,10,6);
  
  myGLCD.print(":", 36, 6);
  myGLCD.print("05", 29, 6);
  //myGLCD.print("2000", 39, 6);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN5])-(VREF/1023*anRead[VOUT5])),2,39,6);
  
  myGLCD.print(":", 65, 6);
  myGLCD.print("06", 58, 6);
  //myGLCD.print("2000", 68, 6);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN6])-(VREF/1023*anRead[VOUT6])),2,68,6);
    
  myGLCD.print(":", 7, 12);
  myGLCD.print("07", 0, 12);
  //myGLCD.print("2000", 10, 12);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN7])-(VREF/1023*anRead[VOUT7])),2,10,12);
  
  myGLCD.print(":", 36, 12);
  myGLCD.print("08", 29, 12);
  //myGLCD.print("2000", 39, 12);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN8])-(VREF/1023*anRead[VOUT8])),2,39,12);
  
  myGLCD.print(":", 65, 12);
  myGLCD.print("09", 58, 12);
  //myGLCD.print("2000", 68, 12);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN9])-(VREF/1023*anRead[VOUT9])),2,68,12);

  myGLCD.print(":", 7, 18);
  myGLCD.print("10", 0, 18);
  //myGLCD.print("2000", 10, 18);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN10])-(VREF/1023*anRead[VOUT10])),2,10,18);
  
  myGLCD.print(":", 36, 18);
  myGLCD.print("11", 29, 18);
  //myGLCD.print("2000", 39, 18);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN11])-(VREF/1023*anRead[VOUT11])),2,39,18);
  
  myGLCD.print(":", 65, 18);
  myGLCD.print("12", 58, 18);
  //myGLCD.print("2000", 68, 18);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN12])-(VREF/1023*anRead[VOUT12])),2,68,18);

  myGLCD.print(":", 7, 24);
  myGLCD.print("13", 0, 24);
  //myGLCD.print("2000", 10, 24);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN13])-(VREF/1023*anRead[VOUT13])),2,10,24);
  
  myGLCD.print(":", 36, 24);
  myGLCD.print("14", 29, 24);
  //myGLCD.print("2000", 39, 24);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN14])-(VREF/1023*anRead[VOUT14])),2,39,24);
  
  myGLCD.print(":", 65, 24);
  myGLCD.print("15", 58, 24);
  //myGLCD.print("2000", 68, 24);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN15])-(VREF/1023*anRead[VOUT15])),2,68,24);
  
  myGLCD.print(":", 7, 30);
  myGLCD.print("16", 0, 30);
  //myGLCD.print("2000", 10, 30);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN16])-(VREF/1023*anRead[VOUT16])),2,10,30);
  
  myGLCD.print(":", 36, 30);
  myGLCD.print("17", 29, 30);
  //myGLCD.print("2000", 39, 30);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN17])-(VREF/1023*anRead[VOUT17])),2,39,30);
  
  myGLCD.print(":", 65, 30);
  myGLCD.print("18", 58, 30);
  //myGLCD.print("2000", 68, 30);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN18])-(VREF/1023*anRead[VOUT18])),2,68,30);

  myGLCD.print(":", 7, 36);
  myGLCD.print("19", 0, 36);
  //myGLCD.print("2000", 10, 36);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN19])-(VREF/1023*anRead[VOUT19])),2,10,36);

  myGLCD.print(":", 65, 36);
  myGLCD.print("20", 58, 36);
  //myGLCD.print("2000", 68, 36);
  myGLCD.printNumF((abs(VREF/1023*anRead[VIN20])-(VREF/1023*anRead[VOUT20])),2,68,36);
  myGLCD.print("VRES [V]", CENTER,42);
  myGLCD.update();
}

void displayBATSTAT()
{
   //myGLCD.drawLine(26,0,26,41);
  //myGLCD.drawLine(27,0,27,41);
  myGLCD.print(":", 7, 0);
  myGLCD.print("01", 0, 0);
  //myGLCD.print("2000", 10, 0);
  myGLCD.print(BATSTATSTR[BATSTAT[1]],10,0);
  
  myGLCD.print(":", 36, 0);
  myGLCD.print("02", 29, 0);
  //myGLCD.print("2000", 39, 0);
  myGLCD.print(BATSTATSTR[BATSTAT[2]],39,0);
  
  myGLCD.print(":", 65, 0);
  myGLCD.print("03", 58, 0);
  //myGLCD.print("2000", 68, 0);
  myGLCD.print(BATSTATSTR[BATSTAT[3]],68,0);
  
  myGLCD.print(":", 7, 6);
  myGLCD.print("04", 0, 6);
  //myGLCD.print("2000", 10, 6);
  myGLCD.print(BATSTATSTR[BATSTAT[4]],10,6);
  
  myGLCD.print(":", 36, 6);
  myGLCD.print("05", 29, 6);
  //myGLCD.print("2000", 39, 6);
  myGLCD.print(BATSTATSTR[BATSTAT[5]],39,6);
  
  myGLCD.print(":", 65, 6);
  myGLCD.print("06", 58, 6);
  //myGLCD.print("2000", 68, 6);
  myGLCD.print(BATSTATSTR[BATSTAT[6]],68,6);
    
  myGLCD.print(":", 7, 12);
  myGLCD.print("07", 0, 12);
  //myGLCD.print("2000", 10, 12);
  myGLCD.print(BATSTATSTR[BATSTAT[7]],10,12);
  
  myGLCD.print(":", 36, 12);
  myGLCD.print("08", 29, 12);
  //myGLCD.print("2000", 39, 12);
  myGLCD.print(BATSTATSTR[BATSTAT[8]],39,12);
  
  myGLCD.print(":", 65, 12);
  myGLCD.print("09", 58, 12);
  //myGLCD.print("2000", 68, 12);
  myGLCD.print(BATSTATSTR[BATSTAT[9]],68,12);

  myGLCD.print(":", 7, 18);
  myGLCD.print("10", 0, 18);
  //myGLCD.print("2000", 10, 18);
  myGLCD.print(BATSTATSTR[BATSTAT[10]],10,18);
  
  myGLCD.print(":", 36, 18);
  myGLCD.print("11", 29, 18);
  //myGLCD.print("2000", 39, 18);
  myGLCD.print(BATSTATSTR[BATSTAT[11]],39,18);
  
  myGLCD.print(":", 65, 18);
  myGLCD.print("12", 58, 18);
  //myGLCD.print("2000", 68, 18);
  myGLCD.print(BATSTATSTR[BATSTAT[12]],68,18);

  myGLCD.print(":", 7, 24);
  myGLCD.print("13", 0, 24);
  //myGLCD.print("2000", 10, 24);
  myGLCD.print(BATSTATSTR[BATSTAT[13]],10,24);
  
  myGLCD.print(":", 36, 24);
  myGLCD.print("14", 29, 24);
  //myGLCD.print("2000", 39, 24);
  myGLCD.print(BATSTATSTR[BATSTAT[14]],39,24);
  
  myGLCD.print(":", 65, 24);
  myGLCD.print("15", 58, 24);
  //myGLCD.print("2000", 68, 24);
  myGLCD.print(BATSTATSTR[BATSTAT[15]],68,24);
  
  myGLCD.print(":", 7, 30);
  myGLCD.print("16", 0, 30);
  //myGLCD.print("2000", 10, 30);
  myGLCD.print(BATSTATSTR[BATSTAT[16]],10,30);
  
  myGLCD.print(":", 36, 30);
  myGLCD.print("17", 29, 30);
  //myGLCD.print("2000", 39, 30);
  myGLCD.print(BATSTATSTR[BATSTAT[17]],39,30);
  
  myGLCD.print(":", 65, 30);
  myGLCD.print("18", 58, 30);
  //myGLCD.print("2000", 68, 30);
  myGLCD.print(BATSTATSTR[BATSTAT[18]],68,30);

  myGLCD.print(":", 7, 36);
  myGLCD.print("19", 0, 36);
  //myGLCD.print("2000", 10, 36);
  myGLCD.print(BATSTATSTR[BATSTAT[19]],10,36);

  myGLCD.print(":", 65, 36);
  myGLCD.print("20", 58, 36);
  //myGLCD.print("2000", 68, 36);
  myGLCD.print(BATSTATSTR[BATSTAT[20]],68,36);
  myGLCD.print("Battery status", CENTER,42);
  myGLCD.update();
}

void displayMAH()
{
   //myGLCD.drawLine(26,0,26,41);
  //myGLCD.drawLine(27,0,27,41);
  myGLCD.print(":", 7, 0);
  myGLCD.print("01", 0, 0);
  //myGLCD.print("2000", 10, 0);
  myGLCD.printNumF(mah[1],0,10,0);
  
  myGLCD.print(":", 36, 0);
  myGLCD.print("02", 29, 0);
  //myGLCD.print("2000", 39, 0);
  myGLCD.printNumF(mah[2],0,39,0);
  
  myGLCD.print(":", 65, 0);
  myGLCD.print("03", 58, 0);
  //myGLCD.print("2000", 68, 0);
  myGLCD.printNumF(mah[3],0,68,0);
  
  myGLCD.print(":", 7, 6);
  myGLCD.print("04", 0, 6);
  //myGLCD.print("2000", 10, 6);
  myGLCD.printNumF(mah[4],0,10,6);
  
  myGLCD.print(":", 36, 6);
  myGLCD.print("05", 29, 6);
  //myGLCD.print("2000", 39, 6);
  myGLCD.printNumF(mah[5],0,39,6);
  
  myGLCD.print(":", 65, 6);
  myGLCD.print("06", 58, 6);
  //myGLCD.print("2000", 68, 6);
  myGLCD.printNumF(mah[6],0,68,6);
    
  myGLCD.print(":", 7, 12);
  myGLCD.print("07", 0, 12);
  //myGLCD.print("2000", 10, 12);
  myGLCD.printNumF(mah[7],0,10,12);
  
  myGLCD.print(":", 36, 12);
  myGLCD.print("08", 29, 12);
  //myGLCD.print("2000", 39, 12);
  myGLCD.printNumF(mah[8],0,39,12);
  
  myGLCD.print(":", 65, 12);
  myGLCD.print("09", 58, 12);
  //myGLCD.print("2000", 68, 12);
  myGLCD.printNumF(mah[9],0,68,12);

  myGLCD.print(":", 7, 18);
  myGLCD.print("10", 0, 18);
  //myGLCD.print("2000", 10, 18);
  myGLCD.printNumF(mah[10],0,10,18);
  
  myGLCD.print(":", 36, 18);
  myGLCD.print("11", 29, 18);
  //myGLCD.print("2000", 39, 18);
  myGLCD.printNumF(mah[11],0,39,18);
  
  myGLCD.print(":", 65, 18);
  myGLCD.print("12", 58, 18);
  //myGLCD.print("2000", 68, 18);
  myGLCD.printNumF(mah[12],0,68,18);

  myGLCD.print(":", 7, 24);
  myGLCD.print("13", 0, 24);
  //myGLCD.print("2000", 10, 24);
  myGLCD.printNumF(mah[13],0,10,24);
  
  myGLCD.print(":", 36, 24);
  myGLCD.print("14", 29, 24);
  //myGLCD.print("2000", 39, 24);
  myGLCD.printNumF(mah[14],0,39,24);
  
  myGLCD.print(":", 65, 24);
  myGLCD.print("15", 58, 24);
  //myGLCD.print("2000", 68, 24);
  myGLCD.printNumF(mah[15],0,68,24);
  
  myGLCD.print(":", 7, 30);
  myGLCD.print("16", 0, 30);
  //myGLCD.print("2000", 10, 30);
  myGLCD.printNumF(mah[16],0,10,30);
  
  myGLCD.print(":", 36, 30);
  myGLCD.print("17", 29, 30);
  //myGLCD.print("2000", 39, 30);
  myGLCD.printNumF(mah[17],0,39,30);
  
  myGLCD.print(":", 65, 30);
  myGLCD.print("18", 58, 30);
  //myGLCD.print("2000", 68, 30);
  myGLCD.printNumF(mah[18],0,68,30);

  myGLCD.print(":", 7, 36);
  myGLCD.print("19", 0, 36);
  //myGLCD.print("2000", 10, 36);
  myGLCD.printNumF(mah[19],0,10,36);

  myGLCD.print(":", 65, 36);
  myGLCD.print("20", 58, 36);
  //myGLCD.print("2000", 68, 36);
  myGLCD.printNumF(mah[20],0,68,36);
  myGLCD.print("Capacity [mAh]", CENTER,42);
  myGLCD.update();
}

void displayMENU(){

    myGLCD.print(">", 0, actualPosition*6-6);
    
    myGLCD.print("CHRG+Q_TST+STRG", 5, 0);
    myGLCD.print("CHRG+Q_TST+CHRG", 5, 6);
    myGLCD.print("CHRG+Q_TST", 5, 12);
    myGLCD.print("CHARGE", 5, 18);
    myGLCD.print("CHRG+STRG", 5, 24);
    myGLCD.print("STORAGE", 5, 30);
    myGLCD.print("QUICK IntResTst", 5, 36);
    myGLCD.print("INFO", 5, 42);
    //myGLCD.print("MENU", CENTER,42);
    myGLCD.update();
}

void askContinue(){
  myGLCD.clrScr();
  myGLCD.print("ARE YOU SURE?", CENTER, 6);
  myGLCD.print("N --> PB or W", CENTER, 18);
  myGLCD.print("Y --> LPB", CENTER, 24);
  myGLCD.update();
  askCont = true;
  timerAskCont = millis();
}

void chargerON(){
  digitalWrite(RELAY,LOW);
  Serial.println("charger ON");
}
void chargerOFF(){
  digitalWrite(RELAY,HIGH);
  Serial.println("charger OFF");
}

int countBatStatOK(){
    int countBatStatOk = 0;
    
    for(int i=1;i<=20;i++)
    {
      if(BATSTAT[i]==1)
      countBatStatOk++;
    }
    Serial.print("Dobrych ogniw: ");
    Serial.println(countBatStatOk);
    return countBatStatOk;
} 

void continueDischarge(int stat_no){
  DateTime now = rtc.now();
  float current = 0.0;
  float vin = 0.0;
  float vout = 0.0;
  long timePassed = 0;
    
  digitalWrite(BATPIN[stat_no], HIGH);


  vin = (VREF/1023*anRead[VINPIN[stat_no]]);
  vout = (VREF/1023*anRead[VOUTPIN[stat_no]]);
  //vout = 0.2;
  current = (vin - vout) / RESVAL[stat_no]; // I=U/R [A]
  //timePassed = (millis()-timerAction);
  timePassed = millis()-timerDisStat[stat_no];
  
  if((vin-vout > 2.5) && ((millis()-timerDischarging)>=ACTION_INTERVAL))
  {

  firstDischEnter = false;
  
  Serial.print(timePassed);
  Serial.print("\t");

  //mah[stat_no] = mah[stat_no] + ((current*1000) * (ACTION_INTERVAL/1000/3600)); // A*1000=mA ; ms/1000/3600=h
  mah[stat_no] = mah[stat_no] + (float)(current*timePassed/3600); // A*1000=mA ; ms/1000/3600=h

  timerDisStat[stat_no] = millis();
  
  Serial.print(stat_no);
  Serial.print("\t");
  Serial.print(String(now.day()) + "-" + String(now.month()) + "-" + String(now.year()) + "_"  + String(now.hour()) + ":" + String(now.minute()) + ":" + String(now.second()));
  Serial.print("\t");
  Serial.print((millis()-timerDischarging)/1000);
  Serial.print("\t");
  Serial.print(vin-vout);
  Serial.print("\t");
  Serial.println(mah[stat_no]);

    /*myFile = SD.open("test.txt", FILE_WRITE);

    if (myFile) {
    myFile.print(stat_no);
    myFile.print("\t");
    myFile.print(String(now.day()) + "-" + String(now.month()) + "-" + String(now.year()) + "_"  + String(now.hour()) + ":" + String(now.minute()) + ":" + String(now.second()));
    myFile.print("\t");
    myFile.print((millis()-timerDischarging)/1000);
    myFile.print("\t");
    myFile.print(vin-vout);
    myFile.print("\t");
    myFile.println(mah[stat_no]);
    myFile.close();
    } else {
    Serial.println("error opening test.txt");
  }*/
  }
  if(firstDischEnter)
  timerDisStat[stat_no] = millis();
}

void stopDischarge(int stat_no){
  digitalWrite(BATPIN[stat_no], LOW);
}


void checkTemp(){
  float temp = 0;
  for(int i=1;i<=20;i++)
  {
    temp = Thermistor(anRead[TEMPPIN[i]]);
    if(temp >= OVERHEAT)
    {

    action = 0;
    chargerOFF();
    charging = false;
    discharging = false;
    firstDischEnter = true;
    chargingComplete = false;
    dischargingComplete = false;
    storageCharging = false;
    for(int i=1;i<=20;i++)
    digitalWrite(BATPIN[i], LOW);
    timerAction = millis();

      
      myGLCD.clrScr();
      myGLCD.printNumF(i,0,CENTER,30);
      myGLCD.print("!!! OVERHEAT !!!", CENTER,42);
      myGLCD.update();
      
      while(1)
      {
      digitalWrite(BUZZER,HIGH);
      delay(100);
      digitalWrite(BUZZER, LOW);
      delay(100);
      }
    }
  }
}

